"""
Helper methods to wrap text in html with semantic coloring
"""

SUCCESS_GREEN = "#00FF00"
ERROR_RED = "#EF5350"
WARN_YELLOW = "#FFFF00"
DEBUG_PURPLE = "#BA68C8"


def element_wrap(text: str, color="white", element="span", italicized=False):
    """Wrap in target element, color, and font style"""
    font_style = "italic" if italicized else "normal"
    return f"<{element} style='color: {color}; font-style: {font_style};'>{text}</{element}>"


def anchor_wrap(url: str, text: str):
    return (
        f"<a href={url} "
        f'style="text-decoration: underline" '
        f'target="_blank" '
        f'rel="noopener noreferrer">'
        f"{text}</a>"
    )


def success_wrap(text: str, element="span", italicized=False):
    return element_wrap(text, SUCCESS_GREEN, element, italicized)


def error_wrap(text: str, element="span", italicized=False):
    return element_wrap(text, ERROR_RED, element, italicized)


def warn_wrap(text: str, element="span", italicized=False):
    return element_wrap(text, WARN_YELLOW, element, italicized)


def debug_wrap(text: str, element="span", italicized=False):
    return element_wrap(text, DEBUG_PURPLE, element, italicized)



"""
In addition to logging, "SandboxReporter" methods will also handle sending formatted html messages to sandbox console
This class is a "logger adapter" that uses duck typing of standard logger signature
"""
import inspect
import logging

from cloudshell.api.cloudshell_api import CloudShellAPISession
from cloudshell.helpers.sandbox_reporter.sandbox_console import SandboxConsole

DEFAULT_STACK_CORRECTION = 3


class SandboxReporter:
    def __init__(self, api: CloudShellAPISession, reservation_id: str, logger: logging.Logger, stack_correction=0):
        """
        Stack correction is when you put sandbox reporter into a wrapper and want the parent function logged
        stack corrections will take effect only for python 3.8 and higher
        """
        self._api = api
        self._reservation_id = reservation_id
        self._logger = logger
        self._console = SandboxConsole(api, reservation_id)
        self._stack_correction = stack_correction

    @property
    def logger(self):
        """
        The underlying logger is always here to use directly
        Can be passed along to lower level functions to avoid unexpected console prints
        (Unless you are debugging and want everything dumped directly to console)
        """
        return self._logger

    @property
    def console(self):
        """
        The console printer can be used directly to print something without logging it
        If you don't care about logging, better to instantiate console independently
        """
        return self._console

    def _is_enabled_for(self, level):
        """
        Is this logger enabled for level 'level'?
        """
        return self.logger.isEnabledFor(level)

    def _log(self, level, msg, *args, **kwargs):
        """
        Delegate a log call to the underlying logger
        Adjust stack level for newer python versions, 3.8 and higher
        """
        sig = inspect.getfullargspec(self.logger._log)  # pylint: disable=protected-access
        if "stacklevel" in sig.args:  # Python 3.8 and above
            total_stacks = DEFAULT_STACK_CORRECTION + self._stack_correction
            self.logger.log(level, msg, *args, stacklevel=total_stacks, **kwargs)
        else:
            self.logger.log(level, msg, *args, **kwargs)

    def debug(self, msg, *args, console_print=True, console_prefix=True, **kwargs):
        """
        Delegate a debug call to the underlying logger
        Prints purple text with contextual prefix to console
        """
        if self._is_enabled_for(logging.DEBUG):
            self._log(logging.DEBUG, msg, *args, **kwargs)
            if console_print:
                if console_prefix:
                    msg = "[DEBUG] - " + msg
                self.console.debug_print(msg)

    def info(self, msg, *args, console_print=True, **kwargs):
        """
        Delegate an info call to the underlying logger
        Neutral color print, optional to set different html element such as H2, H3, etc
        The html signature is slightly different here to avoid un-necessary html wrapping on neutral color prints
        """
        if self._is_enabled_for(logging.INFO):
            self._log(logging.INFO, msg, *args, **kwargs)
            if console_print:
                self.console.sb_print(msg)

    def warning(self, msg, *args, console_print=True, **kwargs):
        """
        Delegate a warning call to the underlying logger
        Print Yellow text to console
        """
        if self._is_enabled_for(logging.WARNING):
            self._log(logging.WARNING, msg, *args, **kwargs)
            if console_print:
                self.console.warn_print(msg)

    def warning_header(self, msg, *args, console_print=True, **kwargs):
        """
        Delegate a warning call to the underlying logger
        Print Yellow text to console
        """
        if self._is_enabled_for(logging.WARNING):
            self._log(logging.WARNING, msg, *args, **kwargs)
            if console_print:
                self.console.warn_print(msg, html_element="h4", italicized=True)

    def error(self, msg, *args, console_print=True, **kwargs):
        """
        Delegate an error call to the underlying logger
        Print Red text to console
        """
        if self._is_enabled_for(logging.ERROR):
            self._log(logging.ERROR, msg, *args, **kwargs)
            if console_print:
                self.console.err_print(msg)

    def exception(self, msg, *args, console_print=True, **kwargs):
        """
        Delegate an exception call to the underlying logger
        Print Red text to console
        """
        if self._is_enabled_for(logging.ERROR):
            self._log(logging.ERROR, msg, *args, exc_info=True, **kwargs)
            if console_print:
                self.console.err_print(msg)

    def critical(self, msg, *args, console_print=True, console_prefix=True, **kwargs):
        """
        Delegate a critical call to the underlying logger.
        Print red text to console with additional contextual prefix
        """
        if self._is_enabled_for(logging.CRITICAL):
            self._log(logging.CRITICAL, msg, *args, **kwargs)
            if console_print:
                if console_prefix:
                    msg = "[CRITICAL] - " + msg
                self.console.err_print(msg)

    def success(self, msg, *args, console_print=True, **kwargs):
        """
        Delegate an info call to the underlying logger
        Neutral color print, optional to set different html element such as H2, H3, etc
        The html signature is slightly different here to avoid un-necessary html wrapping on neutral color prints
        """
        if self._is_enabled_for(logging.INFO):
            self._log(logging.INFO, msg, *args, **kwargs)
            if console_print:
                self.console.success_print(msg)



"""
Methods to format html and send api message to sandbox console
HTML wrap module functions have the semantic color encapsulated
"""
from cloudshell.api.cloudshell_api import CloudShellAPISession
from cloudshell.helpers.sandbox_reporter import html_wrap


class SandboxConsole:
    def __init__(self, api: CloudShellAPISession, reservation_id: str):
        """Store api and sandbox id for api call"""
        self._api = api
        self._reservation_id = reservation_id

    def sb_print(self, message: str):
        """Shorthand alias for printing to sandbox console"""
        self._api.WriteMessageToReservationOutput(self._reservation_id, message)

    def html_print(self, message, color="white", html_element="span", italicized=False):
        """Flexible method for custom html color and elements"""
        self.sb_print(html_wrap.element_wrap(message, color, html_element, italicized))

    def err_print(self, message: str, html_element="span", italicized=False):
        """Print red message to highlight errors"""
        self.sb_print(html_wrap.error_wrap(message, html_element, italicized))

    def success_print(self, message: str, html_element="span", italicized=False):
        """Print green message to highlight passing action"""
        self.sb_print(html_wrap.success_wrap(message, html_element, italicized))

    def warn_print(self, message: str, html_element="span", italicized=False):
        """Print yellow message to highlight important messages"""
        self.sb_print(html_wrap.warn_wrap(message, html_element, italicized))

    def debug_print(self, message: str, html_element="span", italicized=False):
        """Print yellow message to highlight important messages"""
        self.sb_print(html_wrap.debug_wrap(message, html_element, italicized))

    def anchor_tag_print(self, url: str, text: str):
        """For wrapping text in html anchor tag; opens link in new tab by default"""
        self.sb_print(html_wrap.anchor_wrap(url, text))
